/*
    <ClientSample.c>>
ңУǺץ
     ꥨ ͥåȥңУǥ <ͥåȥط饤֥ꥵץ륽>
     Version 0.001, Dec 2002
     ------------------------------------------------------------
     Copyright (C) 2002ǯ naoki kuwamori<naoki@kuwamori.com>

     ܥץϥե꡼եȥǤʤϡFree Software Foundation
     ɽGNU ̸ͭѵΡ֥С󣲡װϤʹߤγƥС
     椫餤줫򤷡ΥС˽äܥץ
     ۤޤѹ뤳ȤǤޤ

    ʳΥ饤󥹤˾ξϡ᡼뤯.
    <naoki@kuwamori.com>


    Laurier Net work RPG server<game network library>
    Version 0.001, Dec 2002
    Copyright (C) 2002 Naoki Kuwamori <naoki@kuwamori.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or any
    later version.

    If you wish another license . 
    mail me <naoki@kuwamori.com> .

*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "libnet.h"
/*ϤäȤ
 * #include "laurel/libnet.h"*/

 int dummyfunc(char *command , char * readbuf , int read_len , int resfd);
 int helthcheck(char *command , char * readbuf , int read_len , int resfd);
 int chdlevel(char *command , char * readbuf , int read_len , int resfd);
 int endreq(char *command , char * readbuf , int read_len , int resfd);
 int shmctlreq(char *command , char * readbuf , int read_len , int resfd);
int nreceivetest(char *command , char * readbuf , int read_len , int resfd);
int sreceivetest(char *command , char * readbuf , int read_len , int resfd);
int lreceivetest(char *command , char * readbuf , int read_len , int resfd);
int mmaptest(char *command , char * readbuf , int read_len , int resfd);
int shmtest(char *command , char * readbuf , int read_len , int resfd);
 
 #define DYMMYMES            "DYMMYMESGE"            /*ߡΥåݻ*/
 #define HELTHCHK            "HELTHCHECK"            /*إ륹å*/
 #define HELTHCHKOK          "HELTHCHKOK"            /*إ륹å*/
 #define DBGLEVEL            "DEBUGLEVEL"            /*ǥХå٥륻å׵*/
 #define ENDREQ              "ENDREQUEST"            /*λ׵*/
 #define SHMCTLREQ			 "SHMCNTLREQ"			/*ͭɣļ׵*/
 #define LAURELLOGSOCNAME    "/tmp/laurellog"
 #define DEFAULTGAMENAME "laureltest"
 #define CONFIGFILENAME "config_file.csv"
 #define LAURELLOGFILENAME  "laurelsampserv.log"
 #define LOGFUNCLEVEL        150

#define SENDTEST1	"SENDTEST01"
#define SENDTEST2	"SENDTEST02"
#define SENDTEST3	"SENDTEST03"
#define SENDTEST4	"SENDTEST04"
#define SENDTEST5	"SENDTEST05"
#define MMAPTESTREQ	"MMAPTEST__"
#define SHMEMTESTREQ	"SHMEMTEST_"
#define NORMALSENDTEST      "NORMSETEST"            /*̾ƥ*/
#define LARGESENDTEST       "LARGESTEST"            /*̾ƥ*/
#define SMALLSENDTEST       "SMALLSTEST"            /*̾ƥ*/





static	int ctlshmid;
RPG_shmunit * ctlshmadder;
char * shmadder;
char * mmapadder;

 
/*TODO ץ
 * ʴǥեեƤ
 * ¤Τ*/
typedef struct{
	char hostname[1024];
	char portnom[1024];
	char UnixSocName[1024];
	char MMapFilePath[1024];
	char Uportnom[1024];
	int  shmID;
}config_file;

/*TODO ץ
 * եեΥƥץ졼Ȥ롣*/
static CCSVTemplate tmpl_templateConfig_file[] = { 
    CSVTRECORD( offsetof( config_file, hostname),CSV_string,1023), 
    CSVTRECORD( offsetof( config_file, portnom),CSV_string,1023), 
    CSVTRECORD( offsetof( config_file, UnixSocName),CSV_string,1023),
    CSVTRECORD( offsetof( config_file, MMapFilePath),CSV_string,1023),
    CSVTRECORD( offsetof( config_file, Uportnom),CSV_string,1023), 
    CSVTRECORD( offsetof( config_file, shmID),CSV_int,0), 
    EndOfCSVTemplate
};


/*Хåؿ򥻥åȤ*/
typedef struct {
	char event[11];             /* number of event */
	int (*func)();  /* function address */
}EVTBL;

/*TODO ץ
 * ٥ȥޥåפ򤳤ˤ
 * ϥ٥ȤФ륳Хåؿ򤳤Ͽ롣*/
EVTBL eventkye[20] = {
         /* ٥             ؿ*/
	{DYMMYMES,              dummyfunc},      /*ߡؿ*/
	{HELTHCHK,              helthcheck},     /*إ륹å׵*/
	{HELTHCHKOK,            helthcheck},     /*إ륹å*/
	{DBGLEVEL,              chdlevel},       /*ǥХå٥ѹ׵*/
	{ENDREQ,                endreq},         /*λ׵*/
	{SHMCTLREQ,             shmctlreq},         /*λ׵*/
	{NORMALSENDTEST,		nreceivetest},
	{LARGESENDTEST,			lreceivetest},
	{SMALLSENDTEST,			sreceivetest},
	{MMAPTESTREQ,			mmaptest},
	{SHMEMTESTREQ,			shmtest},
	{"",                     NULL}
};

char * gamename(char * lgamename){
    static char ggamename[MAXPATHLEN];
    static int flag=0;
    if (flag==0){
        bzero(ggamename,MAXPATHLEN);
        strncpy(ggamename,DEFAULTGAMENAME,MAXPATHLEN);
        flag=1;
    }
    if (lgamename!=NULL) strncpy(ggamename,lgamename,MAXPATHLEN);
    return ggamename;
}

char * logsockname(char * socname){
    static char gsockname[MAXPATHLEN];
    static int flag=0;
    if (flag==0){
        bzero(gsockname,MAXPATHLEN);
        strncpy(gsockname,LAURELLOGSOCNAME,MAXPATHLEN);
        flag=1;
    }
    if (socname!=NULL) strncpy(gsockname,socname,MAXPATHLEN);
    return gsockname;
}


/*ɤ߽ФƥåȤ*/
/*TODO ץ
 * ư˴ؤϤ
 * man getopt_long 򻲾Ȥ뤳*/
int set_opt(int argc, char *argv[]){
    int c;
    while (1)
    {
        int option_index = 0;
        static struct option long_options[] =
        {
           {"version", 0, 0, 'v'},
           {"help", 0, 0, 'h'},
           {"logfilename", 1, 0, 'f'},
           {"logsocname", 1, 0, 's'},
           {"debug", 1, 0, 'd'},
           {"gamename", 1, 0, 'g'},
           {0, 0, 0, 0}
        };
        c = getopt_long (argc, argv, "vhf:s:d:g:",
                    long_options, &option_index);
        if (c == -1) break;
        switch (c)
        {
           case 'v':
                printf ("LaurelRPG server\n");
                exit (1);
             break;
           case 'h':
                printf ("LaurelRPG server\n");
                exit (1);
             break;
           case 'f':
                rpg_setlogfile(optarg);
             break;
           case 's':
                logsockname(optarg);
             break;
           case 'd':
                rpg_dbglvl(atoi(optarg));
             break;
           case 'g':
                gamename(optarg);
             break;

           case '?':
             break;
           default:
             printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }

    if (optind < argc)
    {
     printf ("nonoption ARGVelements: ");
     while (optind < argc)
     printf ("%s ", argv[optind++]);
     printf ("\n");
    }
	return (optind);
}


int main (int argc, char *argv[])
{
    char readbuf[4000];
    int read_len;
    char command[20];
    int resfd;
    int cnt;
    int UNIXserverFD;
    int TCPserverFD;
    int UDPserverFD;
    int ret;
	config_file testval;
	FILE * fp;

    /*ǥեͤ򥻥åȤ*/
    rpg_dbglvl(101);
    rpg_setlogfile(LAURELLOGFILENAME);

    /*ᤷѿ˼Ǽ*/
    set_opt(argc,argv);

    /*ե졼եɤ߹*/

    /*եΥץ*/
    if ((ret=  rpglog(RPGEMERGLEVEL ,"start loging\n" )) < 0){
        printf("logopnen error abort\n");
        exit (0);
    }

/*TODO ץ
 * CSVǤ줿ե졼եɤ߹߽*/
	fp = fopen (CONFIGFILENAME,"r");
	if (fp==NULL){
		printf("file open error\n");
		exit(0);
	}
	while ( fgets( readbuf, sizeof( readbuf ), fp ), !feof( fp ) ) {
		if (readbuf[0]=='#')continue; /*ȤλϹԤκǽʸˡ*/
		memset (&testval , 0 ,sizeof(config_file));
		if (FALSE==GetCSV( &testval, readbuf, tmpl_templateConfig_file )){
			printf("error of read cvs file\n");
		}   
		printf("print nom[%s] string[%s] Mmap[%s]\n", testval.portnom , testval.UnixSocName,testval.MMapFilePath);
		break;
	}

/*TODO ץ*/
    /*ФȤΥå³*/
    ret = rpg_loginit( argv[0] , LAURELLOGSOCNAME);
    printf ("rpg_loginit %s\n",argv[0]);
    if (ret < 0){
        rpglog (RPGEMERGLEVEL,"cant connect log server\n");
    }


/*TODO ץ
 *  * ˥ååȡ̾դѥסˤΥХץ*/
    /*ͥåȥСΥץ(˥ååȤΥץ)*/
    if ((ret = rpg_serveropen( AF_UNIX ,testval.UnixSocName)) < 0){
        char errbuf[1024];
        sprintf (errbuf , "rpg_serveropen error [%s] [%s] [%d]\n",testval.UnixSocName, strerror(errno) , ret);
        rpglog (RPGEMERGLEVEL,errbuf);
        rpg_endproc(RPG_NG);
    }
    UNIXserverFD=ret;
	
/*TODO ץ
 *  * ԣãХåȤΥСץ*/
	rpglog(LOGFUNCLEVEL , "Open TCP server socket\n");	
    if ((ret = rpg_serveropen( AF_INET ,testval.portnom)) < 0){
        char errbuf[1024];
        sprintf (errbuf , "rpg_serveropen error TCP [%s] [%s] [%d]\n",testval.portnom , strerror(errno) , ret);
        rpglog (RPGEMERGLEVEL,errbuf);
        rpg_endproc(1);
    }
    TCPserverFD=ret;

/*TODO ץ
 *  * UDPåȤΥСץ*/
    if ((ret = rpg_serveropen( LIBNET_UDP ,testval.Uportnom)) < 0){
        char errbuf[1024];
        sprintf (errbuf , "rpg_serveropen error UDP [%s] [%s] [%d]\n",testval.Uportnom , strerror(errno) , ret);
        rpglog (RPGEMERGLEVEL,errbuf);
        rpg_endproc(1);
    }
    UDPserverFD=ret;


    /*ͭγ*/
/*TODO ץ
 * ͭΥץ*/
	{
		char * ret1;
		shment shmentry[1];
		shmentry[0].shmNO = testval.shmID;
		shmentry[0].size = 0x1000000;
		ret1 = RPG_shmctl_init( shmentry , 1  , &ctlshmid );
		if (ret1 == NULL){
			rpglog (RPGEMERGLEVEL,"cant open mmmap\n");
			PRG_shmctl_destroy();
			rpg_endproc(RPG_NG);
		}
		(char *)ctlshmadder = ret1;
	}

	shmadder= RPG_shm_attach(testval.shmID);
	if (-1==(int)shmadder){
		rpglog (RPGEMERGLEVEL,"cant attach shm\n");
		rpg_endproc(RPG_NG);
	}
/*TODO ץ
 * ͣͣͭΥץ*/
    (char *)ret = rpg_mopen(testval.MMapFilePath);
    if ((char *)ret == NULL){
        rpglog (RPGEMERGLEVEL,"cant open mmmap\n");
        rpg_endproc(RPG_NG);
    }
    mmapadder = (char *)ret;


/*TODO ץ*/
    /*λΤǥǡˤʤ*/
	rpglog(LOGFUNCLEVEL , "fork child prosses\n");
    ret = fork();
    if (ret > 0){
        if (ret==1){
            rpglog (RPGEMERGLEVEL,"cant fork child prosses\n");
        }
        exit (1);
    }

    /*ᥤ롼*/
    while (1){
        memset (readbuf , 0 , 4000);
        ret = RPG_Poll (command ,  readbuf , 4000, &read_len , 10000 , &resfd );
        readbuf[4000]=0;
        rpglog (1050, "ret %d [%.10s] \n" , ret , command );
        for (cnt=0;cnt<20;cnt++){
            if (eventkye[cnt].func==NULL)break;
                /*ޥʸӤפ*/
            if (strncmp(command ,eventkye[cnt].event,COMMANDLEN)==0){
                /*ХåؿƤӽФ*/
                ret = eventkye[cnt].func(command ,readbuf,read_len , resfd);
            }
        }
    }
    return RPG_OK;
}

/*ߡؿ*/
int dummyfunc(char *command , char * readbuf , int read_len , int resfd){
    rpglog(LOGFUNCLEVEL , "dummyfunc\n");

    return RPG_OK;
}
/*إ륹åؿ׵ξ*/
int helthcheck(char *command , char * readbuf , int read_len , int resfd){
    rpglog(LOGFUNCLEVEL , "helthcheck\n");

    /*ֿ*/
    if (memcmp(HELTHCHK,command,COMMANDLEN)==0){
        rpg_send(resfd,HELTHCHKOK,"",0);
    }
    return RPG_OK;
}
/*ǥХå٥ѹ׵*/
int chdlevel(char *command , char * readbuf , int read_len , int resfd){
    int rev;
    rpglog(LOGFUNCLEVEL , "chdlevel\n");
    /*ӥåǥб*/
    rev = ntohl (*((int *)readbuf));
    rpg_dbglvl(rev);
    return RPG_OK;
}
/*λ׵*/
int endreq(char *command , char * readbuf , int read_len , int resfd){
    rpglog(LOGFUNCLEVEL , "endreq\n");
    rpg_fd_allclose();
    rpglog(-1,"end log file\n");
	exit(0);
}
/*ͭꥳȥɣļؿ*/
int shmctlreq(char *command , char * readbuf , int read_len , int resfd){
    rpglog(LOGFUNCLEVEL , "shmctlreq\n");

	rpg_send(resfd,command,(char *)&ctlshmid,sizeof(int));

	return RPG_OK;
}
int nreceivetest(char *command , char * readbuf , int read_len , int resfd){
	readbuf[read_len]=0;
    rpglog(100 , "nreceivetest %s\n",readbuf);

	return RPG_OK;
}
int lreceivetest(char *command , char * readbuf , int read_len , int resfd){
	int cnt;
	int sum;
    rpglog(100 , "lreceivetest len[%d]\n",read_len);
	if (read_len < 3500){
		rpglog(70 , "lreceivetest:******************** ERROR [%d] The receive length short ********************\n",read_len);
		return RPG_OK;
	}
	for (sum=0,cnt =0;cnt<3500;cnt++){
		sum=sum+readbuf[cnt];
	/*	printf("%c",readbuf[cnt]);*/
		if (readbuf[cnt]==0)printf(" ");
	}
	rpglog(100 , "lreceivetest sum=[%d] receive sum=[%d] [%s]" ,sum,atoi(&(readbuf[3500])),&(readbuf[3500]));
	if (sum!=atoi(&(readbuf[3500]))){
		rpglog(70 , "lreceivetest:******************** ERROR sum miss match ********************\n");
	}

	return RPG_OK;
}
int sreceivetest(char *command , char * readbuf , int read_len , int resfd){
	static int cnt=0;
	int i;
	readbuf[read_len]=0;
    rpglog(1500 , "sreceivetest %d - %s\n",cnt ,readbuf);
	i=atoi(readbuf);
	if (i!=0 && (i)!=(cnt+1)){
		rpglog(70 , "sreceivetest:******************** ERROR [%d]-[%d] The count not continued ********************\n",i, cnt);
	}	
	cnt=i;
	if (i%100 == 0){
		rpglog(100 , "sreceivetest -MARK- %d - %s\n",cnt ,readbuf);
	}
	return RPG_OK;
}

int mmaptest(char *command , char * readbuf , int read_len , int resfd){
	int cnt;
	rpglog(100 , "mmaptest: mmapreadtest %s\n",readbuf);

	if (memcmp(readbuf,"+",1)==0){
		for (cnt =0;cnt<2048;cnt++){
			if ((unsigned char)(mmapadder[cnt])!=cnt%256){
				rpglog(70 , "mmaptest: error mem mispmatch + [%d]-[%d] [%d]\n",mmapadder[cnt],cnt%256,cnt);
			}
		}
	}else{
        for (cnt =0;cnt<2048;cnt++){
			if ((unsigned char)(mmapadder[cnt])!=(2048-cnt)%256){
				rpglog(70 , "mmaptest: error mem mispmatch - [%d]-[%d] [%d]\n",mmapadder[cnt],(2048-cnt)%256,cnt);
			}
		}
		for (cnt =0;cnt<2048;cnt++){
            (unsigned char)(mmapadder[cnt])=cnt%256;
        }
		rpg_send(resfd,MMAPTESTREQ,"+",1);
	}
    return RPG_OK;
}
int shmtest(char *command , char * readbuf , int read_len , int resfd){
	int cnt;
	rpglog(100 , "shmtest: shmreadtest %s\n",readbuf);

	if (readbuf[0]=='+'){
		for (cnt =0;cnt<0x1000000;cnt++){
			if ((unsigned char)(shmadder[cnt])!=cnt%256){
				rpglog(70 , "shmtest: error mem mispmatch + [%d]-[%d] [%d]\n",mmapadder[cnt],cnt%256,cnt);
			}
		}
	}else{
		for (cnt =0;cnt<0x1000000;cnt++){
			if ((unsigned char)(shmadder[cnt])!=(0x1000000-cnt)%256){
				rpglog(70 , "shmtest: error mem mispmatch - [%d]-[%d] [%d]\n",mmapadder[cnt],(2048-cnt)%256,cnt);
			}
		}
        for (cnt =0;cnt< 0x1000000;cnt++){
            (unsigned char)(shmadder[cnt])=cnt%256;
        }
		rpg_send(resfd,SHMEMTESTREQ,"+",1);
	}

    return RPG_OK;
}





